package com.github.gkutiel.filter.x;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;
import static java.text.MessageFormat.format;

import java.io.StringWriter;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.github.gkutiel.filter.Resource;
import com.github.gkutiel.filter.Resource.MimeType;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomWriter;

@MimeType("application/xml")
public abstract class Page<T> extends Resource {

	@Inherited
	@Target(TYPE)
	@Retention(RUNTIME)
	public @interface Xsl {
		String value();
	}

	private static final XStream X_STREAM = new XStream();

	public static String toString(final Document doc) {
		try {
			final StringWriter stringWriter = new StringWriter();
			TransformerFactory.newInstance().newTransformer().transform(new DOMSource(doc), new StreamResult(stringWriter));
			return stringWriter.toString();
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}
	@Override
	public byte[] bytes() {
		try {
			final Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
			X_STREAM.marshal(get(), new DomWriter(doc));
			addXslIfPresent(doc);
			return toString(doc).getBytes();
		} catch (final ParserConfigurationException e) {
			throw new RuntimeException(e);
		}
	}

	private void addXslIfPresent(final Document doc) {
		final Xsl xsl = getClass().getAnnotation(Xsl.class);
		if (xsl == null) return;
		final String stylesheet = format("type=''text/xsl'' href=''{0}''", xsl.value());
		doc.insertBefore(doc.createProcessingInstruction("xml-stylesheet", stylesheet), doc.getDocumentElement());
	}

	protected abstract T get();
}
